/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentDescription.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.descriptor;

import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.ow2.petals.jbi.descriptor.util.XMLUtil;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Component production is used to describe a component installation package.
 *
 * @version $Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ComponentDescription extends ExtensibleJbiElement {

    /**
     * Component type: <code>"binding-component"</code>
     */
    private static final String BC_TYPE = "binding-component";

    /**
     * Compoment type: <code>"service-engine"</code>
     */
    private static final String SE_TYPE = "service-engine";

    /**
     * Component's bootstrap class loading delegation model:
     * <code>"self-first"</code> or <code>"parent-first"</code>.
     */
    private String bootstrapClassLoaderDelegation;

    /**
     * Fully qualified name for bootstrap's <code>Bootstrap</code>
     * implementation.
     */
    private String bootstrapClassName;

    /**
     * Bootstrap Class Path.
     */
    private List<String> bootstrapClassPath;

    /**
     * Component's class loading delegation model: <code>"self-first"</code>
     * or <code>"parent-first"</code>.
     */
    private String componentClassLoaderDelegation;

    /**
     * Fully qualified name for component's <code>Component</code>
     * implementation.
     */
    private String componentClassName;

    /**
     * Component Class Path.
     */
    private List<String> componentClassPath;

    /**
     * Component identification data.
     */
    private Identification identification;

    /**
     * List of Shared Libraries used by this component.
     */
    private List<SharedLibraryList> sharedLibraryList;

    /**
     * Compoment type: SERVICE_ENGINE_TYPE or BINDING_COMPOMENT_TYPE.
     */
    private String type;

    /**
     * Default constructor.
     */
    public ComponentDescription() {
        super();
    }

    @Override
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof ComponentDescription) {
            final ComponentDescription castOther = (ComponentDescription) other;
            bRet = new EqualsBuilder().append(this.bootstrapClassLoaderDelegation,
                    castOther.bootstrapClassLoaderDelegation).append(
                    this.bootstrapClassName, castOther.bootstrapClassName).append(
                    this.bootstrapClassPath, castOther.bootstrapClassPath).append(
                    this.componentClassLoaderDelegation,
                    castOther.componentClassLoaderDelegation).append(
                    this.componentClassName, castOther.componentClassName).append(
                    this.componentClassPath, castOther.componentClassPath).append(
                    this.identification, castOther.identification).append(
                    this.sharedLibraryList, castOther.sharedLibraryList).append(this.type,
                    castOther.type).isEquals();
        } else {
            bRet = false;
        }

        return bRet;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.bootstrapClassLoaderDelegation)
                .append(this.bootstrapClassName).append(this.bootstrapClassPath).append(
                        this.componentClassLoaderDelegation).append(
                                this.componentClassName).append(this.componentClassPath).append(
                                        this.identification).append(this.sharedLibraryList).append(this.type)
                .toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append(
                "bootstrapClassLoaderDelegation",
                this.bootstrapClassLoaderDelegation).append("bootstrapClassName",
                this.bootstrapClassName).append("bootstrapClassPath",
                this.bootstrapClassPath).append("componentClassLoaderDelegation",
                this.componentClassLoaderDelegation).append("componentClassName",
                this.componentClassName).append("componentClassPath",
                this.componentClassPath).append("identification", this.identification)
                .append("sharedLibraryList", this.sharedLibraryList).append("type",
                        this.type).toString();
    }

    /**
     * Generate a XML representation of the component descriptor node
     * @return the XML representation
     */
    @Override
    public Node getNode(final Document jbiDescriptorDocument) {

        final Element componentElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "component");

        final Attr typeAttribut = jbiDescriptorDocument
                            .createAttribute("type");
        typeAttribut.setValue(this.type);
        componentElement.setAttributeNode(typeAttribut);

        if (this.componentClassLoaderDelegation != null
                && !"".equals(this.componentClassLoaderDelegation)) {
            final Attr componentClassLoaderDelegationAttribut = jbiDescriptorDocument
                            .createAttribute("component-class-loader-delegation");
            componentClassLoaderDelegationAttribut.setValue(this.componentClassLoaderDelegation);
            componentElement.setAttributeNode(componentClassLoaderDelegationAttribut);
        }

        if (this.bootstrapClassLoaderDelegation != null
                && !"".equals(this.bootstrapClassLoaderDelegation)) {
            final Attr bootstrapClassLoaderDelegationAttribut = jbiDescriptorDocument
                            .createAttribute("bootstrap-class-loader-delegation");
            bootstrapClassLoaderDelegationAttribut.setValue(this.bootstrapClassLoaderDelegation);
            componentElement.setAttributeNode(bootstrapClassLoaderDelegationAttribut);
        }

        componentElement.appendChild(this.identification.getNode(jbiDescriptorDocument));

        final Element componentClassNameElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "component-class-name");
        componentClassNameElement.setTextContent(this.componentClassName);
        componentElement.appendChild(componentClassNameElement);

        final Element componentClassPathElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "component-class-path");
        for (int i = 0; i < this.componentClassPath.size(); i++) {
            final Element pathElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "path-element");
            pathElement.setTextContent(this.componentClassPath.get(i));
            componentClassPathElement.appendChild(pathElement);
        }
        componentElement.appendChild(componentClassPathElement);

        final Element bootstrapClassNameElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "bootstrap-class-name");
        bootstrapClassNameElement.setTextContent(this.bootstrapClassName);
        componentElement.appendChild(bootstrapClassNameElement);

        final Element bootstrapClassPathElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "bootstrap-class-path");
        for (int i = 0; i < this.bootstrapClassPath.size(); i++) {
            final Element pathElement = jbiDescriptorDocument
                            .createElementNS(XMLUtil.JBI_NAMESPACE, "path-element");
            pathElement.setTextContent(this.bootstrapClassPath.get(i));
            bootstrapClassPathElement.appendChild(pathElement);
        }
        componentElement.appendChild(bootstrapClassPathElement);
        
        if (this.sharedLibraryList != null && this.sharedLibraryList.size() > 0) {
            for (int i = 0; i < this.sharedLibraryList.size(); i++) {
                componentElement.appendChild(this.sharedLibraryList.get(i).getNode(jbiDescriptorDocument));
            }
        }

        // The extensions
        componentElement.appendChild(super.getNode(jbiDescriptorDocument));

        return componentElement;
    }

    //
    // Getters and Setters
    //

    /**
     * Returns the class loading delegation model of component's bootstrap used
     * during component installation. If <code>null</code> the "parent-first"
     * default delegation model must be used.
     *
     * @return <code>String</code> the delegation model:
     *         <code>"parent-first"</code> or <code>"self-first"</code>.
     */
    public final String getBootstrapClassLoaderDelegation() {
        return this.bootstrapClassLoaderDelegation;
    }

    /**
     * Returns the fully-qualified class name for the component's
     * <code>Bootstrap</code> implementation. This is used during component
     * installation.
     *
     * @return <code>String</code> the fully-qualified class name.
     */
    public final String getBootstrapClassName() {
        return this.bootstrapClassName;
    }

    /**
     * Returns the class path used during installation processing
     * (bootstrapping) of the component.
     *
     * @return <code>List&lt;String&gt;</code> the bootstrap class path.
     */
    public final List<String> getBootstrapClassPath() {
        return this.bootstrapClassPath;
    }

    /**
     * Returns the class loading delegation model of this component used during
     * its execution phase. If <code>null</code> the "parent-first" default
     * delegation model must be used.
     *
     * @return <code>String</code> the delegation model:
     *         <code>"parent-first"</code> or <code>"self-first"</code>.
     */
    public final String getComponentClassLoaderDelegation() {
        return this.componentClassLoaderDelegation;
    }

    /**
     * Returns the fully-qualified class name for the component's
     * <code>Component</code> implementation. This is used during component's
     * execution phase of its lifecycle.
     *
     * @return <code>String</code> the fully-qualified class name.
     */
    public final String getComponentClassName() {
        return this.componentClassName;
    }

    /**
     * Returns the class path used after the component is installed. This class
     * path, along with shared-libraries, constitutes the full class path
     * avaiable to the component through its default class loader context as
     * provided by JBI.
     *
     * @return <code>List&lt;String&gt;</code> the component class path.
     */
    public final List<String> getComponentClassPath() {
        return this.componentClassPath;
    }

    /**
     * Returns component's identification data.
     *
     * @return <code>Identification</code> the component's identification.
     */
    public final Identification getIdentification() {
        return this.identification;
    }

    /**
     * Returns a list of shared libraries (by name and optional version) used by
     * the component. The implementation may use the version information to
     * verify that the shared library version declared by the component matches
     * the version of the shared library installed in the JBI environment.
     *
     * @return <code>List&lt;SharedLibraryList&gt;</code> used by this component.
     */
    public final List<SharedLibraryList> getSharedLibraryList() {
        return this.sharedLibraryList;
    }

    /**
     * The component type string representation specifying whether this is a
     * service engine or a binding component.
     *
     * @return <code>String</code> the component type.
     */
    public final String getType() {
        return this.type;
    }

    /**
     * Returns <code>true</code> if this component type is a Binding
     * Component. Otherwise returns <code>false</code>.
     */
    public final boolean isBindingComponent() {
        return ComponentDescription.BC_TYPE.equals(this.type);
    }

    /**
     * Returns <code>true</code> if this component type is a Service Engine.
     * Otherwise returns <code>false</code>.
     */
    public final boolean isServiceEngine() {
        return ComponentDescription.SE_TYPE.equals(this.type);
    }

    /**
     * Set the Bootstrap class loading delegation model.
     *
     * @param delegationModel
     *            The delegation model to set: <code>"parent-first"</code> or
     *            <code>"self-first"</code>.
     */
    public final void setBootstrapClassLoaderDelegation(
            final String delegationModel) {
        this.bootstrapClassLoaderDelegation = delegationModel;
    }

    /**
     * Set the class name of component's <code>Bootstrap</code>
     * implementation.
     *
     * @param className
     *            <code>String</code> fully-qualified class name.
     */
    public final void setBootstrapClassName(final String className) {
        this.bootstrapClassName = className;
    }

    /**
     * Set the class path used by this component's <code>Bootstrap</code>
     * during installation process.
     *
     * @param classPath
     *            <code>List&lt;String&gt;</code> the bootstrap class path.
     */
    public final void setBootstrapClassPath(final List<String> classPath) {
        this.bootstrapClassPath = classPath;
    }

    /**
     * Set the Component class loading delegation model.
     *
     * @param delegationModel
     *            the delegation model to set: <code>"parent-first"</code> or
     *            <code>"self-first"</code>.
     */
    public final void setComponentClassLoaderDelegation(
            final String delegationModel) {
        this.componentClassLoaderDelegation = delegationModel;
    }

    /**
     * Set the class name of this component implementation.
     *
     * @param className
     *            <code>String</code> fully-qualified class name.
     */
    public final void setComponentClassName(final String className) {
        this.componentClassName = className;
    }

    /**
     * Set the class path used by this component during its execution
     * processing.
     *
     * @param classPath
     *            <code>List&lt;String&gt;></code> the component class path.
     */
    public final void setComponentClassPath(final List<String> classPath) {
        this.componentClassPath = classPath;
    }

    /**
     * Set the component identification data.
     *
     * @param identification
     *            The identification to set.
     */
    public final void setIdentification(final Identification identification) {
        this.identification = identification;
    }

    /**
     * Set shared library list used by this component.
     *
     * @param sharedLibraryList
     *            <code>List&lt;SharedLibraryList&gt;</code> shared library list to
     *            set.
     */
    public final void setSharedLibraryList(
            final List<SharedLibraryList> sharedLibraryList) {

        this.sharedLibraryList = sharedLibraryList;
    }

    /**
     * Set the component type: <code>{@link ComponentDescription#SE_TYPE}</code>
     * or <code>{@link ComponentDescription#BC_TYPE}</code>.
     *
     * @param type
     *            <code>String</code> the component type to set.
     */
    public final void setType(final String type) {
        this.type = type;
    }
}
