/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceUnit.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.descriptor;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.ow2.petals.jbi.descriptor.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * The Service Unit production is used to describe a service component within a
 * service assembly deployment package.
 *
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ServiceUnit extends ExtensibleJbiElement {

    /**
     * Service Unit identification data.
     */
    private Identification identification;

    /**
     * Name of the artifact archive file.
     */
    private String targetArtifactsZip;

    /**
     * Component name.
     */
    private String targetComponentName;

    /**
     * Default constructor.
     */
    public ServiceUnit() {
        super();
    }

    @Override
    public boolean equals(final Object other) {
        boolean bRet;
        if (other instanceof ServiceUnit) {
            final ServiceUnit castOther = (ServiceUnit) other;
            bRet = new EqualsBuilder().append(this.identification,
                    castOther.identification).append(this.targetArtifactsZip,
                    castOther.targetArtifactsZip).append(this.targetComponentName,
                    castOther.targetComponentName).isEquals();
        } else {
            bRet = false;
        }

        return bRet;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(this.identification).append(
                this.targetArtifactsZip).append(this.targetComponentName).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("identification",
                this.identification)
                .append("targetArtifactsZip", this.targetArtifactsZip).append(
                        "targetComponentName", this.targetComponentName).toString();
    }

    /**
     * Generate a XML representation of the service unit descriptor node
     *
     * @return the XML representation
     */
    @Override
    public Node getNode(final Document jbiDescriptorDocument) {

        final Element serviceUnitElement = jbiDescriptorDocument
                .createElementNS(XMLUtil.JBI_NAMESPACE, "service-unit");

        serviceUnitElement.appendChild(this.identification
                .getNode(jbiDescriptorDocument));

        final Element targetElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "target");
        serviceUnitElement.appendChild(targetElement);

        final Element zipElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "artifacts-zip");
        zipElement.setTextContent(this.targetArtifactsZip);
        targetElement.appendChild(zipElement);

        final Element componentElement = jbiDescriptorDocument.createElementNS(
                XMLUtil.JBI_NAMESPACE, "component-name");
        componentElement.setTextContent(this.targetComponentName);
        targetElement.appendChild(componentElement);

        return serviceUnitElement;
    }

    //
    // Getters and Setters
    //

    /**
     * Returns service unit's identification data.
     *
     * @return <code>Identification</code> the SU's identification.
     */
    public final Identification getIdentification() {
        return this.identification;
    }

    /**
     * Returns the name of the artifact archive file within the deployment
     * package that will be deployed.
     *
     * @return <code>String</code> the artifact archive file.
     */
    public final String getTargetArtifactsZip() {
        return this.targetArtifactsZip;
    }

    /**
     * Returns the name of the component that will have the <i>artifacts zip</i>
     * deployed to it.
     *
     * @return <code>String</code> the component name.
     */
    public final String getTargetComponentName() {
        return this.targetComponentName;
    }

    /**
     * Set the service unit identification data.
     *
     * @param identification
     *            The identification to set.
     */
    public final void setIdentification(final Identification identification) {
        this.identification = identification;
    }

    /**
     * Set the target artifacts archive file within the deployment package.
     *
     * @param targetZip
     *            <code>String</code> the archive file.
     */
    public final void setTargetArtifactsZip(final String targetZip) {
        this.targetArtifactsZip = targetZip;
    }

    /**
     * Set the target component name to which the artifacts archive file is to
     * be deployed.
     *
     * @param targetCompName
     *            <code>String</code> the component name.
     */
    public final void setTargetComponentName(final String targetCompName) {
        this.targetComponentName = targetCompName;
    }

}
